<?php
namespace HCM\Geral\Repository\Esocial;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;

class S2298Repository {

    protected  $connection;

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
    }
    
    public function getItensParaEnvioS2298($params) {
        $sql = <<<SQL
SELECT VH.NRVINCULOM as NRCHAVE,
       DECODE(ENV.ENVIOS,NULL,'I','A') as IDOPERACAO,
       VH.DTMESCOMPETENC as DTMESCOMPETENC,
       VM.NRVINCULOM
  FROM GPE_VINCULOM VM
  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                      AND VH.NRVINCULOM = VM.NRVINCULOM
                      AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH H1
                                                WHERE H1.NRVINCULOM = VH.NRVINCULOM
                                                  AND H1.NRORG = VH.NRORG
                                                  AND H1.DTMESCOMPETENC <= :P_DTMESCOMPETENC)
  /*QUERY PARA VERIFICAR SE É INCLUSÃO OU ALTERAÇÃO DO REGISTRO*/
  LEFT JOIN (SELECT EC.NRORG, EC.NRCHAVE, COUNT(*) AS ENVIOS
               FROM ESO_ENVIOCADASTROS EC
               JOIN ESO_EVENTO EV ON EV.NREVENTO = EC.NREVENTO
                                 AND EV.NMEVENTO = 'S-2298'
              WHERE EC.NRORG = :P_NRORG
                AND EC.IDSTATUS = 'C' /*QUE ESTIVER CONCLUIDO*/
                AND EC.NRESTRUTURAM = :P_NRESTRUTURAM
                AND EC.NRTPAMBIENTE = :P_NRTPAMBIENTE
              GROUP BY EC.NRORG, EC.NRCHAVE) ENV ON ENV.NRCHAVE = VH.NRVINCULOM
 WHERE VM.NRORG = :P_NRORG
   AND VH.NRESTRUTLEGAL = :P_NRESTRUTURAM
   AND (VM.DTRESCISAOVINC IS NULL OR VM.DTRESCISAOVINC >= :P_DTMESCOMPETENC)
   AND VM.IDTPREINTEGRACAO IS NOT NULL
   AND (VM.NRVINCULOM = :P_NRVINCULOM OR :P_NRVINCULOM IS NULL)
   AND (VM.DTRETORNO >= :P_DTREFNAOPERIOD OR :P_DTREFNAOPERIOD IS NULL)
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRESTRUTURAM" => isset($params['NRESTRUTURAM']) ? $params['NRESTRUTURAM'] : null,
            "P_NRTPAMBIENTE" => isset($params['NRTPAMBIENTE']) ? $params['NRTPAMBIENTE'] : null,
            "P_NRVINCULOM" => isset($params['NRVINCULOM']) ? $params['NRVINCULOM'] : null,
            "P_DTREFNAOPERIOD" => isset($params['DTREFNAOPERIOD']) ? $params['DTREFNAOPERIOD'] : null,
        );
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getDadosS2298($params) {
        try {
            $dados = $this->getDadosPrincipais($params['NRENVIO'], $params['NRCHAVE'], $params['NRESTRUTURAM']);
            return $dados;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function getDadosPrincipais($nrenvio, $nrchave, $nrestruturam) {
        $sql = <<<SQL
/*getDadosPrincipaisS2298*/
SELECT VM.NRVINCULOM CHAVE,
       'ID'||DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','')||RPAD(CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END,14,'0')||TO_CHAR(SYSDATE, 'YYYYMMDDHH24MISS')||SUBSTR('00000'||:P_NRENVIO,-5,100) ID,
       --ideEmpregador
       DECODE(PAR.CDTIPOINSCRICAO,'CNPJ','1','CPF','2','') TPINSC,
       CASE WHEN PAR.CDTIPOINSCRICAO = 'CPF' OR EMH.CDNATUJURI IN ('1015','1040','1074','1163') THEN PAR.NRINSCRICAOPARC ELSE SUBSTR(PAR.NRINSCRICAOPARC,1,8) END NRINSC,    
       --ideVinculo
       REPLACE(REPLACE(PH.NRCPFPESSOA,'.'),'-') AS CPFTRAB,
       NVL(VH.CDMATRICULAESOCIAL, VM.NRVINCULOM) AS matricula, 
       --infoReintegr
       VM.IDTPREINTEGRACAO AS tpReint,
       VM.NRPROCJUD AS nrProcJud,
       VM.NRLEIANISTIA AS nrLeiAnistia,       
       TO_CHAR(VM.DTRETORNO,'YYYY-MM-DD')  AS dtEfetRetorno,
       TO_CHAR(VM.DTEFEITOFINANCEIRO,'YYYY-MM-DD') AS dtEfeito
  FROM GPE_VINCULOM VM
  /*Dados do Empregado - Vínculo*/
  JOIN GPE_VINCULOH VH
    ON VH.NRVINCULOM = VM.NRVINCULOM
   AND VH.NRORG      = VM.NRORG
   AND VH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                             WHERE GPE_VINCULOH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_VINCULOH.DTMESCOMPETENC >= ADD_MONTHS(TRUNC(TO_DATE(:P_DTMESCOMPETENC),'YEAR'),-12)
                               AND GPE_VINCULOH.NRVINCULOM = VM.NRVINCULOM
                               AND GPE_VINCULOH.NRORG = VM.NRORG)
  /*Dados do Empregado - Pessoa*/                               
  JOIN GPE_PESSOA PE
    ON PE.NRPESSOA = VM.NRPESSOA
   AND PE.NRORG    = VM.NRORG
  JOIN GPE_PESSOAH PH
    ON PH.NRPESSOA = PE.NRPESSOA
   AND PH.NRORG    = PE.NRORG
   AND PH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH
                             WHERE GPE_PESSOAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                               AND GPE_PESSOAH.NRPESSOA = PE.NRPESSOA
                               AND GPE_PESSOAH.NRORG = PE.NRORG)
  JOIN PARCNEGOCIO PN
    ON PN.NRPARCNEGOCIO = PE.NRPARCNEGOCIO
   AND PN.NRORG         = PE.NRORG                               
  /*Empregador*/
  JOIN ESTRUTURAH EMH ON EMH.NRORG = :P_NRORG
                     AND EMH.NRESTRUTURAM = :P_NRESTRUTURAM
                     AND EMH.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                               WHERE ESTRUTURAH.DTMESCOMPETENC <= :P_DTMESCOMPETENC
                                                 AND ESTRUTURAH.NRESTRUTURAM = EMH.NRESTRUTURAM
                                                 AND ESTRUTURAH.NRORG = EMH.NRORG)
  JOIN PARCNEGOCIO PAR ON PAR.NRPARCNEGOCIO = EMH.NRPARCNEGOCIO
                      AND PAR.NRORG = EMH.NRORG
 WHERE VM.NRORG = :P_NRORG
   AND VM.NRVINCULOM = :P_NRCHAVE
SQL;
        $params = array(
            "P_NRORG" => $this->hcmParameters->getNrorgTrabalho(),
            "P_DTMESCOMPETENC" => $this->hcmParameters->getDtCompetencia(),
            "P_NRENVIO" => $nrenvio,
            "P_NRCHAVE" => $nrchave,
            "P_NRESTRUTURAM" => $nrestruturam
        );
        return $this->connection->fetchAssoc($sql, $params);
    }

}