<?php
namespace HCM\PTO\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\Util;

class GerenciarOcorrenciaFrenquenciaController extends Crud {

    protected $dataSourceGetGerenOcorFreq = 'getGerenOcorFreq';
    protected $dataSourceSetGerenOcorFreq = 'ptoProgOcorrencia_setGerenOcorFreq';
    protected $dataSourceGetTipoProgOcor  = 'ptoTipoProgOcorrencia_getTipoOcorrenciaFrequencia';
    protected $dataSourceGetMotivoProgOcor= 'ptoMotivoOcorFreq_getMotivoOcorFreq';
 
    protected $parameterBagColumns = array('NRORG', 'NRORG_PADRAO', 'NRPERIODOAPURACAO', 'P_DTMESCOMPETENC', 'NRVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST');
    
    protected const COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER = ['NRVINCULOM_LIST', 'NRESTRUTLEGAL_LIST', 'NRESTRUTGEREN_LIST'];
    
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag){
        $this->parameterBag    = $parameterBag;
        parent::__construct($dataSourceManager);
    }

    public function getGerenOcorFreq (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetGerenOcorFreq);
            $filterData = Util::createNewFilterColumnConditionsToCommaImploded($filterData, self::COLLUMNS_TO_BE_ALTERED_BEFORE_APPLY_FILTER);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
   
    
    public function getMotivoProgOcor (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $this->parameterBagColumns = array('NRORG');
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetMotivoProgOcor);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function getTipoProgOcor (DTO\Request\Filter $request, DTO\Response $response) {
        try{
            $filterData = $request->getFilterCriteria();
            $filterData->setDataSourceName($this->dataSourceGetTipoProgOcor);
            $newFilterData = $this->populateParameterBag($filterData);
            $dataSet = $this->dataSourceManager->findBy($newFilterData); 
            $response->addDataSet($dataSet);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function setGerenOcorFreq (DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = $this->dataSourceSetGerenOcorFreq;
        try{
            foreach($request->getDataSet()->getRows() as $row) {
                $row['DTULTATU'] = $row['DTATUAL'];
                $row['CDOPERULTATU'] = $row['CDOPERADOR_LOG'];
                $row['NRORGULTATU'] = $row['NRORGTRAB'];
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteGerenOcorFreq (DTO\Request\Row $row, DTO\Response $response) {
        try{
            $dataSet = new DataSet($this->dataSourceSetGerenOcorFreq, array($row->getRow()));
            $deletedRows = $this->dataSourceManager->delete($dataSet);

            $response->addDataSet(new DataSet($this->dataSourceSetGerenOcorFreq, array('type' => 'success', $deletedRows)));
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }

    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }

        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
}